/*
 * aconvert.cpp
 *
 *  Created on: 26. 3. 2014
 *	  Author: Martin Zak
 */

#include <tclap/CmdLine.h>
#include <global/GlobalData.h>
#include <measure>

#include <exception/CommonException.h>
#include <lexer/Lexer.h>
#include <parser/Parser.h>

#include <factory/XmlDataFactory.hpp>

int main ( int argc, char * argv[] ) {
	try {
		common::GlobalData::argc = argc;
		common::GlobalData::argv = argv;

		TCLAP::CmdLine cmd ( "Convert binary", ' ', "0.01" );

		TCLAP::SwitchArg automaton_from_string ( "", "automaton_from_string", "Convert automaton from string representation" );
		cmd.add ( automaton_from_string );

		TCLAP::SwitchArg automaton_to_string ( "", "automaton_to_string", "Convert automaton to string representation" );
		cmd.add ( automaton_to_string );

		TCLAP::SwitchArg automaton_to_dot ( "", "automaton_to_dot", "Convert automaton to dot representation" );
		cmd.add ( automaton_to_dot );

		TCLAP::SwitchArg automaton_to_gastex ( "", "automaton_to_gastex", "Convert automaton to gastex representation" );
		cmd.add ( automaton_to_gastex );

		TCLAP::SwitchArg automaton_to_tikz ( "", "automaton_to_tikz", "Convert automaton to tikz representation" );
		cmd.add ( automaton_to_tikz );

		TCLAP::SwitchArg grammar_from_string ( "", "grammar_from_string", "Convert grammar from string representation" );
		cmd.add ( grammar_from_string );

		TCLAP::SwitchArg grammar_to_string ( "", "grammar_to_string", "Convert grammar to string representation" );
		cmd.add ( grammar_to_string );

		TCLAP::SwitchArg regexp_from_string ( "", "regexp_from_string", "Convert regexp from string representation" );
		cmd.add ( regexp_from_string );

		TCLAP::SwitchArg regexp_to_string ( "", "regexp_to_string", "Convert regexp to string representation" );
		cmd.add ( regexp_to_string );

		TCLAP::SwitchArg string_from_string ( "", "string_from_string", "Convert string from string representation" );
		cmd.add ( string_from_string );

		TCLAP::SwitchArg string_to_string ( "", "string_to_string", "Convert string to string representation" );
		cmd.add ( string_to_string );

		TCLAP::SwitchArg tree_from_string ( "", "tree_from_string", "Convert tree from string representation" );
		cmd.add ( tree_from_string );

		TCLAP::SwitchArg tree_to_string ( "", "tree_to_string", "Convert tree to string representation" );
		cmd.add ( tree_to_string );

		TCLAP::ValueArg < std::string > input ( "i", "input", "Input file", false, "-", "file" );
		cmd.add ( input );

		TCLAP::SwitchArg measure ( "m", "measure", "Measure times", false );
		cmd.add ( measure );

		TCLAP::SwitchArg verbose ( "v", "verbose", "Be verbose", false );
		cmd.add ( verbose );

		cmd.parse ( argc, argv );

		if ( verbose.isSet ( ) )
			common::GlobalData::verbose = true;

		if ( measure.isSet ( ) )
			common::GlobalData::measure = true;

		cli::Environment environment;
		environment.setBinding ( "stdin", input.getValue ( ) );
		environment.setBinding ( "stdout", "-" );

		measurements::start ( "Overal", measurements::Type::OVERALL );
		measurements::start ( "Input read", measurements::Type::AUXILIARY );

		std::string inputCli;
		if ( automaton_from_string.getValue ( ) || grammar_from_string.getValue ( ) || regexp_from_string.getValue ( ) || string_from_string.getValue ( ) || tree_from_string.getValue ( ) )
			inputCli = "execute cli::builtin::ReadFile #stdin > $input";
		else if ( automaton_to_string.getValue ( ) || automaton_to_dot.getValue ( ) || automaton_to_gastex.getValue ( ) || automaton_to_tikz.getValue ( ) || grammar_to_string.getValue ( ) || regexp_to_string.getValue ( ) || string_to_string.getValue ( ) || tree_to_string.getValue ( ) )
			inputCli = "execute <#stdin > $input";
		else
			throw exception::CommonException("Invalid format specified");

		cli::Parser parser = cli::Parser ( cli::Lexer ( inputCli ) );
		parser.parse ( )->run ( environment );

		measurements::end ( );
		measurements::start ( "Algorithm", measurements::Type::MAIN );

		std::string algoCli;
		if ( automaton_from_string.getValue ( ) )
			algoCli = "execute string::Parse @ automaton::Automaton ^ $input > $output";
		else if ( grammar_from_string.getValue ( ) )
			algoCli = "execute string::Parse @ grammar::Grammar ^ $input > $output";
		else if ( regexp_from_string.getValue ( ) )
			algoCli = "execute string::Parse @ regexp::RegExp ^ $input > $output";
		else if ( string_from_string.getValue ( ) )
			algoCli = "execute string::Parse @ string::String ^ $input > $output";
		else if ( tree_from_string.getValue ( ) )
			algoCli = "execute string::Parse @ tree::Tree ^ $input > $output";
		else if ( automaton_to_string.getValue ( ) )
			algoCli = "execute string::Compose $input > $output";
		else if ( automaton_to_dot.getValue ( ) )
			algoCli = "execute convert::DotConverter $input > $output";
		else if ( automaton_to_gastex.getValue ( ) )
			algoCli = "execute convert::GasTexConverter $input > $output";
		else if ( automaton_to_tikz.getValue ( ) )
			algoCli = "execute convert::TikZConverter $input > $output";
		else if ( grammar_to_string.getValue ( ) )
			algoCli = "execute string::Compose $input > $output";
		else if ( regexp_to_string.getValue ( ) )
			algoCli = "execute string::Compose $input > $output";
		else if ( string_to_string.getValue ( ) )
			algoCli = "execute string::Compose $input > $output";
		else if ( tree_to_string.getValue ( ) )
			algoCli = "execute string::Compose $input > $output";
		else
			throw exception::CommonException("Invalid format specified");

		parser = cli::Parser ( cli::Lexer ( algoCli ) );
		parser.parse ( )->run ( environment );

		measurements::end ( );
		measurements::start ( "Output write", measurements::Type::AUXILIARY );

		std::string outputCli;
		if ( automaton_from_string.getValue ( ) || grammar_from_string.getValue ( ) || regexp_from_string.getValue ( ) || string_from_string.getValue ( ) || tree_from_string.getValue ( ) )
			outputCli = "execute $output >#stdout";
		else if ( automaton_to_string.getValue ( ) || automaton_to_dot.getValue ( ) || automaton_to_gastex.getValue ( ) || automaton_to_tikz.getValue ( ) || grammar_to_string.getValue ( ) || regexp_to_string.getValue ( ) || string_to_string.getValue ( ) || tree_to_string.getValue ( ) )
			outputCli = "execute cli::builtin::WriteFile #stdout $output";
		else
			throw exception::CommonException("Invalid format specified");

		parser = cli::Parser ( cli::Lexer ( outputCli ) );
		parser.parse ( )->run ( environment );

		measurements::end ( );
		measurements::end ( );

		if ( measure.getValue ( ) ) common::Streams::measure << measurements::results ( ) << std::endl;

		return 0;
	} catch ( const exception::CommonException & exception ) {
		factory::XmlDataFactory::toStdout ( exception );
		return 1;
	} catch ( const TCLAP::ArgException & exception ) {
		common::Streams::err << exception.error ( ) << std::endl;
		return 2;
	} catch ( const std::exception & exception ) {
		common::Streams::err << "Exception caught: " << exception.what ( ) << std::endl;
		return 3;
	} catch ( ... ) {
		common::Streams::err << "Unknown exception caught." << std::endl;
		return 127;
	}
}
