#include <registry/Registry.h>
#include <common/CastHelper.h>
#include <exception>

namespace abstraction {

std::shared_ptr < abstraction::OperationAbstraction > CastHelper::eval ( const std::shared_ptr < abstraction::OperationAbstraction > & param, const std::string & type, bool move ) {
	if ( abstraction::Registry::isCastNoOp ( type, param->getReturnType ( ) ) ) {
		return param;
	}

	std::shared_ptr < abstraction::OperationAbstraction > res = abstraction::Registry::getCastAbstraction ( type, param->getReturnType ( ) );
	if ( ! res->attachInput ( param, 0, move ) )
		throw std::invalid_argument ( "Can't connect param at 0 of cast to " + type + " with result of type " + param->getReturnType ( ) + "." );
	if ( ! res->eval ( ) )
		throw std::invalid_argument ( "Eval of cast to " + type + " failed." );

	if ( abstraction::Registry::hasNormalize ( res->getReturnType ( ) ) ) {
		std::shared_ptr < abstraction::OperationAbstraction > normalized = abstraction::Registry::getNormalizeAbstraction ( res->getReturnType ( ) );
		if ( ! normalized->attachInput ( res, 0, true ) )
			throw std::invalid_argument ( "Can't connect param at 0 of normalize of cast to " + type + " with result of type " + res->getReturnType ( ) + "." );
		if ( ! normalized->eval ( ) )
			throw std::invalid_argument ( "Eval of normalize of cast to " + type + " failed." );

		res = normalized;
	}

	return res;
}

} /* namespace abstraction */
