/*
 * ToGrammarRightRG.cpp
 *
 *  Created on: 9. 2. 2014
 *      Author: Tomas Pecka
 */

#include "ToGrammarRightRG.h"
#include <common/createUnique.hpp>
#include <registration/AlgoRegistration.hpp>

namespace automaton {

namespace convert {

grammar::RightRG < > ToGrammarRightRG::convert(const automaton::NFA < > & automaton) {
	ext::map<DefaultStateType, DefaultSymbolType> nonterminalMap;

	const DefaultStateType& initState = automaton.getInitialState();
	DefaultSymbolType initSymbol { initState };

	grammar::RightRG < > grammar(initSymbol);
	nonterminalMap.insert(std::make_pair(initState, initSymbol));

	grammar.setTerminalAlphabet(automaton.getInputAlphabet());

	for(const auto& state : automaton.getStates()) {
		if(state == initState)
			continue;

		DefaultSymbolType nt = common::createUnique(DefaultSymbolType(state), grammar.getTerminalAlphabet(), grammar.getNonterminalAlphabet());
		grammar.addNonterminalSymbol(nt);
		nonterminalMap.insert(std::make_pair(state, nt));
	}

	// step 2 - create set of P in G
	for(const auto& transition : automaton.getTransitions()) {
		const DefaultStateType& from = transition.first.first;
		const DefaultSymbolType& input = transition.first.second;
		for(const auto& to : transition.second) {
			grammar.addRule(nonterminalMap.find(from)->second, ext::make_pair(input, nonterminalMap.find(to)->second)); // 2a
			if(automaton.getFinalStates().count(to)) // 2b
				grammar.addRule(nonterminalMap.find(from)->second, input);
		}
	}

	// step 3 - set start symbol of G
	grammar.setInitialSymbol(nonterminalMap.find(automaton.getInitialState())->second);

	// step 4
	if(automaton.getFinalStates().count(automaton.getInitialState()))
		grammar.setGeneratesEpsilon(true); // okay this feature breaks algorithm but simplifies the code actually :))

	return grammar;
}

grammar::RightRG < > ToGrammarRightRG::convert(const automaton::DFA<>& automaton) {
	ext::map<DefaultStateType, DefaultSymbolType> nonterminalMap;

	const DefaultStateType& initState = automaton.getInitialState();
	DefaultSymbolType initSymbol { initState };

	grammar::RightRG < > grammar(initSymbol);
	nonterminalMap.insert(std::make_pair(initState, initSymbol));

	grammar.setTerminalAlphabet(automaton.getInputAlphabet());

	for(const auto& state : automaton.getStates()) {
		if(state == initState)
			continue;

		DefaultSymbolType nt = common::createUnique(DefaultSymbolType(state), grammar.getTerminalAlphabet(), grammar.getNonterminalAlphabet());
		grammar.addNonterminalSymbol(nt);
		nonterminalMap.insert(std::make_pair(state, nt));
	}

	// step 2 - create set of P in G
	for(const auto& transition : automaton.getTransitions()) {
		const DefaultStateType& from = transition.first.first;
		const DefaultSymbolType& input = transition.first.second;
		const DefaultStateType& to = transition.second;

		grammar.addRule(nonterminalMap.find(from)->second, ext::make_pair(input, nonterminalMap.find(to)->second)); // 2a
		if(automaton.getFinalStates().count(to)) // 2b
			grammar.addRule(nonterminalMap.find(from)->second, input);
	}

	// step 3 - set start symbol of G
	grammar.setInitialSymbol(nonterminalMap.find(automaton.getInitialState())->second);

	// step 4
	if(automaton.getFinalStates().count(automaton.getInitialState()))
		grammar.setGeneratesEpsilon(true); // okay this feature breaks algorithm but simplifies the code actually :))

	return grammar;
}

auto ToGrammarRightRGNFA = registration::AbstractRegister < ToGrammarRightRG, grammar::RightRG < >, const automaton::NFA < > & > ( ToGrammarRightRG::convert );
auto ToGrammarRightRGDFA = registration::AbstractRegister < ToGrammarRightRG, grammar::RightRG < >, const automaton::DFA < > & > ( ToGrammarRightRG::convert );

} /* namespace convert */

} /* namespace automaton */
