/*
 * ToRegExpAlgebraic.h
 *
 *  Created on: 11. 2. 2014
 *	  Author: Tomas Pecka
 */

#ifndef AUTOMATON_TO_REG_EXP_ALGEBRAIC_H_
#define AUTOMATON_TO_REG_EXP_ALGEBRAIC_H_

#include <regexp/unbounded/UnboundedRegExp.h>
#include <automaton/FSM/EpsilonNFA.h>
#include <automaton/FSM/MultiInitialStateNFA.h>
#include <automaton/FSM/NFA.h>
#include <automaton/FSM/DFA.h>

namespace automaton {

namespace convert {

// http://cs.stackexchange.com/questions/2016/how-to-convert-finite-automata-to-regular-expressions
// https://github.com/ferno/greenery/blob/bcc0a136335edbe94cd7725fc6e8cce0268d850c/fsm.py

/**
 * Converts FA to RE using Brzozowski's algebraic method using right regular equations.
 * Source : Melichar 2.122
 */
class ToRegExpAlgebraic {
public:
	/**
	 * Performs conversion.
	 * @return regular expression equivalent to input automaton.
	 */
	static regexp::UnboundedRegExp < > convert(const automaton::EpsilonNFA < > & automaton);
	static regexp::UnboundedRegExp < > convert(const automaton::MultiInitialStateNFA < > & automaton);
	static regexp::UnboundedRegExp < > convert(const automaton::NFA < > & automaton);
	static regexp::UnboundedRegExp < > convert(const automaton::DFA < > & automaton);
};

} /* namespace convert */

} /* namespace automaton */

#endif /* AUTOMATON_TO_REG_EXP_ALGEBRAIC_H_ */
