/*
 * Determinize.h
 *
 *  Created on: 16. 1. 2014
 *	  Author: Jan Vesely
 */

#ifndef DETERMINIZE_H_
#define DETERMINIZE_H_

#include <alib/set>

#include <automaton/PDA/InputDrivenNPDA.h>
#include <automaton/PDA/InputDrivenDPDA.h>
#include <automaton/PDA/RealTimeHeightDeterministicNPDA.h>
#include <automaton/PDA/RealTimeHeightDeterministicDPDA.h>
#include <automaton/PDA/VisiblyPushdownNPDA.h>
#include <automaton/PDA/VisiblyPushdownDPDA.h>
#include <automaton/FSM/NFA.h>
#include <automaton/FSM/DFA.h>
#include <automaton/FSM/MultiInitialStateNFA.h>
#include <automaton/TA/NFTA.h>
#include <automaton/TA/DFTA.h>

#include <automaton/transform/PDAToRHPDA.h>
#include <automaton/transform/RHPDAToPDA.h>

#include <automaton/PDA/NPDA.h>
#include <automaton/PDA/DPDA.h>
#include <automaton/PDA/SinglePopNPDA.h>
#include <automaton/PDA/SinglePopDPDA.h>

#include <automaton/TM/OneTapeDTM.h>

namespace automaton {

namespace determinize {

/**
 * Class for running determinization algorithm on fsm.
 */
class Determinize {
public:
	template < class SymbolType, class StateType >
	static automaton::DFA < SymbolType, StateType > determinize ( const automaton::DFA < SymbolType, StateType > & nfa );

	/**
	 * @param nfsm nondeterministic final-state machine given for determinization
	 * @return DFA
	 * Runs determinization algorithm on nondeterministic fsm given in constructor.
	 */
	template < class SymbolType, class StateType >
	static automaton::DFA < SymbolType, ext::set < StateType > > determinize ( const automaton::NFA < SymbolType, StateType > & nfa );

	template < class SymbolType, class StateType >
	static automaton::DFA < SymbolType, ext::set < StateType > > determinize ( const automaton::MultiInitialStateNFA < SymbolType, StateType > & nfa );

	template < class SymbolType, class RankType, class StateType >
	static automaton::DFTA < SymbolType, RankType, StateType > determinize ( const automaton::DFTA < SymbolType, RankType, StateType > & nfta );

	template < class SymbolType, class RankType, class StateType >
	static automaton::DFTA < SymbolType, RankType, ext::set < StateType > > determinize ( const automaton::NFTA < SymbolType, RankType, StateType > & nfta );

	template < class InputSymbolType, class PushdownSymbolType, class StateType >
	static automaton::InputDrivenDPDA < InputSymbolType, PushdownSymbolType, StateType > determinize ( const automaton::InputDrivenDPDA < InputSymbolType, PushdownSymbolType, StateType > & dpda );

	template < class InputSymbolType, class PushdownSymbolType, class StateType >
	static automaton::InputDrivenDPDA < InputSymbolType, PushdownSymbolType, ext::set < StateType > > determinize ( const automaton::InputDrivenNPDA < InputSymbolType, PushdownSymbolType, StateType > & npda );

	template < class InputSymbolType, class PushdownStoreSymbolType, class StateType >
	static automaton::VisiblyPushdownDPDA < InputSymbolType, PushdownStoreSymbolType, StateType > determinize ( const automaton::VisiblyPushdownDPDA < InputSymbolType, PushdownStoreSymbolType, StateType > & nondeterministic );

	template < class InputSymbolType, class PushdownStoreSymbolType, class StateType >
	static automaton::VisiblyPushdownDPDA < InputSymbolType, ext::pair < ext::set < ext::pair < StateType, StateType > >, InputSymbolType >, ext::set < ext::pair < StateType, StateType > > > determinize ( const automaton::VisiblyPushdownNPDA < InputSymbolType, PushdownStoreSymbolType, StateType > & nondeterministic );

	template < class InputSymbolType, class EpsilonType, class PushdownStoreSymbolType, class StateType >
	static automaton::RealTimeHeightDeterministicDPDA < InputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > determinize ( const automaton::RealTimeHeightDeterministicDPDA < InputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > & nondeterministic );

	template < class InputSymbolType, class EpsilonType, class PushdownStoreSymbolType, class StateType >
	static automaton::RealTimeHeightDeterministicDPDA < InputSymbolType, EpsilonType, ext::pair < ext::set < ext::pair < StateType, StateType > >, ext::variant < EpsilonType, InputSymbolType > >, ext::set < ext::pair < StateType, StateType > > > determinize ( const automaton::RealTimeHeightDeterministicNPDA < InputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > & nondeterministic );

	template < class InputSymbolType, class EpsilonType, class PushdownStoreSymbolType, class StateType >
	static automaton::SinglePopDPDA < InputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > determinize ( const automaton::SinglePopDPDA < InputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > & dpda );

	template < class InputSymbolType, class EpsilonType, class PushdownStoreSymbolType, class StateType >
	static automaton::DPDA < InputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > determinize ( const automaton::DPDA < InputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > & dpda );

	template < class InputSymbolType, class EpsilonType, class PushdownStoreSymbolType, class StateType >
	static automaton::DPDA < InputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > determinize ( const automaton::NPDA < InputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > & dpda );

	template < class SymbolType, class StateType >
	static automaton::OneTapeDTM < SymbolType, StateType > determinize ( const automaton::OneTapeDTM < SymbolType, StateType > & nfta );
};

template < class SymbolType, class StateType >
automaton::DFA < SymbolType, StateType > Determinize::determinize ( const automaton::DFA < SymbolType, StateType > & automaton ) {
	return automaton;
}

template < class SymbolType, class RankType, class StateType >
automaton::DFTA < SymbolType, RankType, StateType > Determinize::determinize ( const automaton::DFTA < SymbolType, RankType, StateType > & automaton ) {
	return automaton;
}

template < class InputSymbolType, class PushdownSymbolType, class StateType >
automaton::InputDrivenDPDA < InputSymbolType, PushdownSymbolType, StateType > Determinize::determinize ( const automaton::InputDrivenDPDA < InputSymbolType, PushdownSymbolType, StateType > & automaton ) {
	return automaton;
}

template < class InputSymbolType, class PushdownStoreSymbolType, class StateType >
automaton::VisiblyPushdownDPDA < InputSymbolType, PushdownStoreSymbolType, StateType > Determinize::determinize ( const automaton::VisiblyPushdownDPDA < InputSymbolType, PushdownStoreSymbolType, StateType > & automaton ) {
	return automaton;
}

template < class InputSymbolType, class EpsilonType, class PushdownStoreSymbolType, class StateType >
automaton::RealTimeHeightDeterministicDPDA < InputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > Determinize::determinize ( const automaton::RealTimeHeightDeterministicDPDA < InputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > & automaton ) {
	return automaton;
}

template < class InputSymbolType, class EpsilonType, class PushdownStoreSymbolType, class StateType >
automaton::SinglePopDPDA < InputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > Determinize::determinize ( const automaton::SinglePopDPDA < InputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > & automaton ) {
	return automaton;
}

template < class InputSymbolType, class EpsilonType, class PushdownStoreSymbolType, class StateType >
automaton::DPDA < InputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > Determinize::determinize ( const automaton::DPDA < InputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > & automaton ) {
	return automaton;
}

template < class InputSymbolType, class EpsilonType, class PushdownStoreSymbolType, class StateType >
automaton::DPDA < InputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > Determinize::determinize ( const automaton::NPDA < InputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > & automaton ) {
	automaton::RealTimeHeightDeterministicNPDA < InputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > rhpda = automaton::PDAToRHPDA::convert(automaton);
	automaton::RealTimeHeightDeterministicDPDA < InputSymbolType, EpsilonType, ext::pair < ext::set < ext::pair < StateType, StateType > >, ext::variant < EpsilonType, InputSymbolType > >, ext::set < ext::pair < StateType, StateType > > > dpda = Determinize::determinize(rhpda);
	automaton::RealTimeHeightDeterministicDPDA < > norm = core::normalize < decltype ( dpda ) >::eval ( std::move ( dpda ) );
	return automaton::RHPDAToPDA::convert ( norm );
}

template < class SymbolType, class StateType >
automaton::OneTapeDTM < SymbolType, StateType > Determinize::determinize ( const automaton::OneTapeDTM < SymbolType, StateType > & automaton ) {
	return automaton;
}

} /* namespace determinize */

} /* namespace automaton */

#include "DeterminizeNFAPart.hxx"
#include "DeterminizeIDPDAPart.hxx"
#include "DeterminizeNFTAPart.hxx"
#include "DeterminizeVPAPart.hxx"
#include "DeterminizeRHDPDAPart.hxx"

#endif /* DETERMINIZE_H_ */
