/*
 * Translate.h
 *
 *  Created on: 12. 5. 2016
 *      Author: Jakub Doupal
 */

#ifndef _AUTOMATON_TRANSLATE_H__
#define _AUTOMATON_TRANSLATE_H__

#include <string/LinearString.h>

#include "Run.h"
#include <automaton/PDA/NPDTA.h>

#include <alib/deque>
#include <alib/set>

namespace automaton {

namespace run {

class Translate {
public:
	/**
	 * Performs conversion.
	 * @return left regular grammar equivalent to source automaton.
	 */
	template < class InputSymbolType, class OutputSymbolType, class EpsilonType, class PushdownStoreSymbolType, class StateType >
	static ext::set < string::LinearString < OutputSymbolType > > translate ( const automaton::NPDTA < InputSymbolType, OutputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > & automaton, const string::LinearString < InputSymbolType > & string );

};

template < class InputSymbolType, class OutputSymbolType, class EpsilonType, class PushdownStoreSymbolType, class StateType >
ext::set < string::LinearString < OutputSymbolType > > Translate::translate ( const automaton::NPDTA < InputSymbolType, OutputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > & automaton, const string::LinearString < InputSymbolType > & string ) {
	ext::tuple < bool, ext::set < StateType >, ext::set < ext::vector < OutputSymbolType > > > res = Run::calculateStates ( automaton, string );
	ext::set < string::LinearString < OutputSymbolType > > strings;

	for ( const auto & rawString : std::get < 2 > ( res ) )
		strings . insert ( string::LinearString < OutputSymbolType > ( rawString ) );

	return strings;
}

} /* namespace run */

} /* namespace automaton */

#endif /* _AUTOMATON_TRANSLATE_H__ */
