/*
 * RandomizeGrammar.h
 *
 *  Created on: 9. 2. 2017
 *	  Author: Jan Travnicek
 */

#ifndef GRAMMAR_RANDOMIZE_H_
#define GRAMMAR_RANDOMIZE_H_

#include <grammar/Regular/LeftLG.h>
#include <grammar/Regular/LeftRG.h>
#include <grammar/Regular/RightLG.h>
#include <grammar/Regular/RightRG.h>
#include <grammar/ContextFree/CFG.h>

#include <common/Permutation.hpp>

namespace grammar {

namespace generate {

class RandomizeGrammar {
public:
	template < class SymbolType >
	static grammar::LeftRG < SymbolType > randomize ( const grammar::LeftRG < SymbolType > & gram );
	template < class SymbolType >
	static grammar::LeftLG < SymbolType > randomize ( const grammar::LeftLG < SymbolType > & gram );
	template < class SymbolType >
	static grammar::RightRG < SymbolType > randomize ( const grammar::RightRG < SymbolType > & gram );
	template < class SymbolType >
	static grammar::RightLG < SymbolType > randomize ( const grammar::RightLG < SymbolType > & gram );
	template < class SymbolType >
	static grammar::CFG < SymbolType > randomize ( const grammar::CFG < SymbolType > & gram );

};

template < class SymbolType >
grammar::LeftRG < SymbolType > RandomizeGrammar::randomize ( const grammar::LeftRG < SymbolType > & gram ) {
	ext::map < SymbolType, SymbolType > symbolPermutationMap = common::Permutation::permutationMap ( gram.getNonterminalAlphabet ( ) );

	grammar::LeftRG < SymbolType > res ( symbolPermutationMap.find ( gram.getInitialSymbol ( ) )->second );

	res.setNonterminalAlphabet ( gram.getNonterminalAlphabet ( ) );
	res.setTerminalAlphabet ( gram.getTerminalAlphabet ( ) );

	for ( const std::pair < SymbolType, ext::set < ext::variant < SymbolType, ext::pair < SymbolType, SymbolType > > > > & rule : gram.getRules ( ) )
		for ( const ext::variant < SymbolType, ext::pair < SymbolType, SymbolType > > & rhs : rule.second )
			if ( rhs.template is < SymbolType > ( ) )
				res.addRule ( symbolPermutationMap.find ( rule.first )->second, rhs );
			else
				res.addRule ( symbolPermutationMap.find ( rule.first )->second, ext::make_pair ( symbolPermutationMap.find ( rhs.template get < ext::pair < SymbolType, SymbolType > > ( ).first )->second, rhs.template get < ext::pair < SymbolType, SymbolType > > ( ).second ) );

	return res;
}

template < class SymbolType >
grammar::LeftLG < SymbolType > RandomizeGrammar::randomize ( const grammar::LeftLG < SymbolType > & gram ) {
	ext::map < SymbolType, SymbolType > symbolPermutationMap = common::Permutation::permutationMap ( gram.getNonterminalAlphabet ( ) );

	grammar::LeftLG < SymbolType > res ( symbolPermutationMap.find ( gram.getInitialSymbol ( ) )->second );

	res.setNonterminalAlphabet ( gram.getNonterminalAlphabet ( ) );
	res.setTerminalAlphabet ( gram.getTerminalAlphabet ( ) );

	for ( const std::pair < SymbolType, ext::set < ext::variant < ext::vector < SymbolType >, ext::pair < SymbolType, ext::vector < SymbolType > > > > > & rule : gram.getRules ( ) )
		for ( const ext::variant < ext::vector < SymbolType >, ext::pair < SymbolType, ext::vector < SymbolType > > > & rhs : rule.second )
			if ( rhs.template is < ext::vector < SymbolType > > ( ) )
				res.addRule ( symbolPermutationMap.find ( rule.first )->second, rhs );
			else
				res.addRule ( symbolPermutationMap.find ( rule.first )->second, ext::make_pair ( symbolPermutationMap.find ( rhs.template get < ext::pair < SymbolType, ext::vector < SymbolType > > > ( ).first )->second, rhs.template get < ext::pair < SymbolType, ext::vector < SymbolType > > > ( ).second ) );

	return res;
}

template < class SymbolType >
grammar::RightRG < SymbolType > RandomizeGrammar::randomize ( const grammar::RightRG < SymbolType > & gram ) {
	ext::map < SymbolType, SymbolType > symbolPermutationMap = common::Permutation::permutationMap ( gram.getNonterminalAlphabet ( ) );

	grammar::RightRG < SymbolType > res ( symbolPermutationMap.find ( gram.getInitialSymbol ( ) )->second );

	res.setNonterminalAlphabet ( gram.getNonterminalAlphabet ( ) );
	res.setTerminalAlphabet ( gram.getTerminalAlphabet ( ) );

	for ( const std::pair < SymbolType, ext::set < ext::variant < SymbolType, ext::pair < SymbolType, SymbolType > > > > & rule : gram.getRules ( ) )
		for ( const ext::variant < SymbolType, ext::pair < SymbolType, SymbolType > > & rhs : rule.second )
			if ( rhs.template is < SymbolType > ( ) )
				res.addRule ( symbolPermutationMap.find ( rule.first )->second, rhs );
			else
				res.addRule ( symbolPermutationMap.find ( rule.first )->second, ext::make_pair ( rhs.template get < ext::pair < SymbolType, SymbolType > > ( ).first, symbolPermutationMap.find ( rhs.template get < ext::pair < SymbolType, SymbolType > > ( ).second )->second ) );

	return res;
}

template < class SymbolType >
grammar::RightLG < SymbolType > RandomizeGrammar::randomize ( const grammar::RightLG < SymbolType > & gram ) {
	ext::map < SymbolType, SymbolType > symbolPermutationMap = common::Permutation::permutationMap ( gram.getNonterminalAlphabet ( ) );

	grammar::RightLG < SymbolType > res ( symbolPermutationMap.find ( gram.getInitialSymbol ( ) )->second );

	res.setNonterminalAlphabet ( gram.getNonterminalAlphabet ( ) );
	res.setTerminalAlphabet ( gram.getTerminalAlphabet ( ) );

	for ( const std::pair < SymbolType, ext::set < ext::variant < ext::vector < SymbolType >, ext::pair < ext::vector < SymbolType >, SymbolType > > > > & rule : gram.getRules ( ) )
		for ( const ext::variant < ext::vector < SymbolType >, ext::pair < ext::vector < SymbolType >, SymbolType > > & rhs : rule.second )
			if ( rhs.template is < ext::vector < SymbolType > > ( ) )
				res.addRule ( symbolPermutationMap.find ( rule.first )->second, rhs );
			else
				res.addRule ( symbolPermutationMap.find ( rule.first )->second, ext::make_pair ( rhs.template get < ext::pair < ext::vector < SymbolType >, SymbolType > > ( ).first, symbolPermutationMap.find ( rhs.template get < ext::pair < ext::vector < SymbolType >, SymbolType > > ( ).second )->second ) );

	return res;
}

template < class SymbolType >
grammar::CFG < SymbolType > RandomizeGrammar::randomize ( const grammar::CFG < SymbolType > & gram ) {
	ext::map < SymbolType, SymbolType > symbolPermutationMap = common::Permutation::permutationMap ( gram.getNonterminalAlphabet ( ) );

	grammar::CFG < SymbolType > res ( symbolPermutationMap.find ( gram.getInitialSymbol ( ) )->second );

	res.setNonterminalAlphabet ( gram.getNonterminalAlphabet ( ) );
	res.setTerminalAlphabet ( gram.getTerminalAlphabet ( ) );

	for ( const std::pair < SymbolType, ext::set < ext::vector < SymbolType > > > & rule : gram.getRules ( ) )
		for ( const ext::vector < SymbolType > & rhs : rule.second )
			res.addRule ( symbolPermutationMap.find ( rule.first )->second, ext::transform < SymbolType > ( rhs, [ & ] ( const SymbolType & elem ) {
						if ( gram.getNonterminalAlphabet ( ).count ( elem ) )
							return symbolPermutationMap.find ( elem )->second;
						else
							return elem;
					} ) );

	return res;
}

} /* namespace generate */

} /* namespace grammar */

#endif /* GRAMMAR_RANDOMIZE_H_ */
