/*
 * Rename.h
 *
 *  Created on: Sep 27, 2016
 *      Author: Tomas Pecka
 */

#include "Rename.h"

#include <alib/algorithm>
#include <alib/map>
#include <alib/vector>

#include <alib/hexavigesimal>
#include <registration/AlgoRegistration.hpp>

namespace grammar {

namespace simplify {

grammar::RightRG < > Rename::rename ( const grammar::RightRG < > & rrg ) {
	int counter = 0;

	ext::map < DefaultSymbolType, DefaultSymbolType > renamingData;

	for ( const DefaultSymbolType & symbol : rrg.getNonterminalAlphabet ( ) )
		renamingData.insert ( std::make_pair ( symbol, DefaultSymbolType ( ext::toBase26 ( counter++ ) ) ) );

	grammar::RightRG < > result ( renamingData.find ( rrg.getInitialSymbol ( ) )->second );
	result.setGeneratesEpsilon ( rrg.getGeneratesEpsilon ( ) );
	result.setTerminalAlphabet ( rrg.getTerminalAlphabet ( ) );

	for ( const DefaultSymbolType & symbol : rrg.getNonterminalAlphabet ( ) )
		result.addNonterminalSymbol ( renamingData.find ( symbol )->second );

	for ( const std::pair < const DefaultSymbolType, ext::set < ext::variant < DefaultSymbolType, ext::pair < DefaultSymbolType, DefaultSymbolType > > > > & rule : rrg.getRules ( ) )
		for ( const ext::variant < DefaultSymbolType, ext::pair < DefaultSymbolType, DefaultSymbolType > > & rhs : rule.second )
			if ( rhs.is < DefaultSymbolType > ( ) ) {
				result.addRule ( renamingData.find ( rule.first )->second, rhs.get < DefaultSymbolType > ( ) );
			} else {
				const ext::pair < DefaultSymbolType, DefaultSymbolType > & realRhs = rhs.get < ext::pair < DefaultSymbolType, DefaultSymbolType > > ( );
				result.addRule ( renamingData.find ( rule.first )->second, ext::make_pair ( realRhs.first, renamingData.find ( realRhs.second )->second ) );
			}

	return result;
}

auto RenameRightRG = registration::AbstractRegister < Rename, grammar::RightRG < >, const grammar::RightRG < > & > ( Rename::rename );

grammar::LeftRG < > Rename::rename ( const grammar::LeftRG < > & lrg ) {
	int counter = 0;

	ext::map < DefaultSymbolType, DefaultSymbolType > renamingData;

	for ( const DefaultSymbolType & symbol : lrg.getNonterminalAlphabet ( ) )
		renamingData.insert ( std::make_pair ( symbol, DefaultSymbolType ( ext::toBase26 ( counter++ ) ) ) );

	grammar::LeftRG < > result ( renamingData.find ( lrg.getInitialSymbol ( ) )->second );
	result.setGeneratesEpsilon ( lrg.getGeneratesEpsilon ( ) );
	result.setTerminalAlphabet ( lrg.getTerminalAlphabet ( ) );

	for ( const DefaultSymbolType & symbol : lrg.getNonterminalAlphabet ( ) )
		result.addNonterminalSymbol ( renamingData.find ( symbol )->second );

	for ( const std::pair < const DefaultSymbolType, ext::set < ext::variant < DefaultSymbolType, ext::pair < DefaultSymbolType, DefaultSymbolType > > > > & rule : lrg.getRules ( ) )
		for ( const ext::variant < DefaultSymbolType, ext::pair < DefaultSymbolType, DefaultSymbolType > > & rhs : rule.second )
			if ( rhs.is < DefaultSymbolType > ( ) ) {
				result.addRule ( renamingData.find ( rule.first )->second, rhs.get < DefaultSymbolType > ( ) );
			} else {
				const ext::pair < DefaultSymbolType, DefaultSymbolType > & realRhs = rhs.get < ext::pair < DefaultSymbolType, DefaultSymbolType > > ( );
				result.addRule ( renamingData.find ( rule.first )->second, ext::make_pair ( renamingData.find ( realRhs.first )->second, realRhs.second ) );
			}

	return result;
}

auto RenameLeftRG = registration::AbstractRegister < Rename, grammar::LeftRG < >, const grammar::LeftRG < > & > ( Rename::rename );

} /* namespace simplify */

} /* namespace grammar */
