#include "RegExpEpsilonTest.h"

#include "regexp/properties/RegExpEpsilon.h"

#include <factory/StringDataFactory.hpp>
#include <regexp/string/UnboundedRegExp.h>

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( RegExpEpsilonTest, "regexp" );
CPPUNIT_TEST_SUITE_REGISTRATION( RegExpEpsilonTest );

void RegExpEpsilonTest::setUp() {
}

void RegExpEpsilonTest::tearDown() {
}

void RegExpEpsilonTest::testRegExpEpsilon() {
	{
		std::string input = "#E + ( (a #E) + a*)";
		regexp::UnboundedRegExp < > re = factory::StringDataFactory::fromString (input);

		CPPUNIT_ASSERT(regexp::properties::RegExpEpsilon::languageContainsEpsilon(re));
	}
	{
		std::string input = "( a* )( a* )";
		regexp::UnboundedRegExp < > re = factory::StringDataFactory::fromString (input);

		CPPUNIT_ASSERT(regexp::properties::RegExpEpsilon::languageContainsEpsilon(re));
	}
	{
		std::string input = "a + #0";
		regexp::UnboundedRegExp < > re = factory::StringDataFactory::fromString (input);

		CPPUNIT_ASSERT(! regexp::properties::RegExpEpsilon::languageContainsEpsilon(re));
	}
	{
		std::string input = "#E + a #E + a*";
		regexp::UnboundedRegExp < > re = factory::StringDataFactory::fromString (input);

		CPPUNIT_ASSERT(regexp::properties::RegExpEpsilon::languageContainsEpsilon(re));
	}
	{
		std::string input = "a* a*";
		regexp::UnboundedRegExp < > re = factory::StringDataFactory::fromString (input);

		CPPUNIT_ASSERT(regexp::properties::RegExpEpsilon::languageContainsEpsilon(re));
	}
	{
		std::string input = "a s d #E + #E #0";
		regexp::UnboundedRegExp < > re = factory::StringDataFactory::fromString (input);

		CPPUNIT_ASSERT(! regexp::properties::RegExpEpsilon::languageContainsEpsilon(re));
	}
	{
		std::string input = "a + #0";
		regexp::UnboundedRegExp < > re = factory::StringDataFactory::fromString (input);

		CPPUNIT_ASSERT(! regexp::properties::RegExpEpsilon::languageContainsEpsilon(re));
	}
}
