/*
 * Deque.h
 *
 * Created on: Apr 1, 2013
 * Author: Jan Travnicek
 */

#ifndef OBJECTS_DEQUE_H_
#define OBJECTS_DEQUE_H_

#include <alib/deque>
#include <alib/string>
#include <sstream>

#include "ContainerBase.h"

#include <object/UniqueObject.h>
#include <object/ObjectFactory.h>

#include <core/normalize.hpp>

namespace container {

/**
 * Basic container from which are derived all other containers.
 * Contains reason why the container occurred.
 */
template < class ElementType = object::Object >
class ObjectsDeque final : public ext::deque < ElementType >, public ContainerBase {
public:
	explicit ObjectsDeque ( ext::deque < ElementType > );
	explicit ObjectsDeque ( );

	virtual ContainerBase * clone ( ) const &;

	virtual ContainerBase * clone ( ) &&;

	virtual int compare ( const ObjectBase & other ) const {
		if ( ext::type_index ( typeid ( * this ) ) == ext::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return ext::type_index ( typeid ( * this ) ) - ext::type_index ( typeid ( other ) );
	}

	virtual int compare ( const ObjectsDeque & other ) const;

	virtual void operator >>( std::ostream & os ) const;

	virtual explicit operator std::string ( ) const;

	virtual object::ObjectBase * inc ( ) &&;
};

template < class ElementType >
ObjectsDeque < ElementType >::ObjectsDeque ( ext::deque < ElementType > raw ) : ext::deque < ElementType > ( std::move ( raw ) ) {
}

template < class ElementType >
ObjectsDeque < ElementType >::ObjectsDeque ( ) : ext::deque < ElementType > ( ) {
}

template < class ElementType >
ContainerBase * ObjectsDeque < ElementType >::clone ( ) const & {
	return new ObjectsDeque ( * this );
}

template < class ElementType >
ContainerBase * ObjectsDeque < ElementType >::clone ( ) && {
	return new ObjectsDeque ( std::move ( * this ) );
}

template < class ElementType >
int ObjectsDeque < ElementType >::compare ( const ObjectsDeque & other ) const {
	static ext::compare < ext::deque < ElementType > > comp;

	return comp ( static_cast < const ext::deque < ElementType > & > ( * this ), static_cast < const ext::deque < ElementType > & > ( other ) );
}

template < class ElementType >
void ObjectsDeque < ElementType >::operator >>( std::ostream & os ) const {
	os << "(ObjectsDeque " << static_cast < const ext::deque < ElementType > & > ( * this ) << ")";
}

template < class ElementType >
ObjectsDeque < ElementType >::operator std::string ( ) const {
	std::stringstream ss;
	ss << * this;
	return std::move ( ss ).str ( );
}

template < class ElementType >
object::ObjectBase* ObjectsDeque < ElementType >::inc() && {
	return new object::UniqueObject(object::Object(std::move(*this)), primitive::Integer(0));
}

} /* namespace container */

namespace core {

template < class ElementType >
struct normalize < container::ObjectsDeque < ElementType > > {
	static ext::deque < object::Object > raw ( ext::deque < ElementType > && source ) {
		ext::deque < object::Object > res;

		for ( ElementType & element : source )
			res.push_back ( object::ObjectFactory::make ( std::move ( element ) ) );

		return res;
	}

	static container::ObjectsDeque < > eval ( container::ObjectsDeque < ElementType > && value ) {
		return container::ObjectsDeque < > ( raw ( std::move ( value ) ) );
	}
};

} /* namespace core */

#endif /* OBJECTS_DEQUE_H_ */
