/*
 * Pair.h
 *
 * Created on: Apr 1, 2013
 * Author: Jan Travnicek
 */

#ifndef OBJECTS_PAIR_H_
#define OBJECTS_PAIR_H_

#include <alib/pair>
#include <alib/string>
#include <sstream>

#include "ContainerBase.h"

#include <object/UniqueObject.h>
#include <object/ObjectFactory.h>

#include <core/normalize.hpp>

namespace container {

/**
 * Basic container from which are derived all other containers.
 * Contains reason why the container occured.
 */
template < class FirstType = object::Object, class SecondType = object::Object >
class ObjectsPair final : public ext::pair < FirstType, SecondType >, public ContainerBase {
public:
	explicit ObjectsPair ( FirstType firstObject, SecondType secondObject );
	explicit ObjectsPair ( ext::pair < FirstType, SecondType > );

	virtual ContainerBase * clone ( ) const &;

	virtual ContainerBase * clone ( ) &&;

	virtual int compare ( const ObjectBase & other ) const {
		if ( ext::type_index ( typeid ( * this ) ) == ext::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return ext::type_index ( typeid ( * this ) ) - ext::type_index ( typeid ( other ) );
	}

	virtual int compare ( const ObjectsPair & other ) const;

	virtual void operator >>( std::ostream & os ) const;

	virtual explicit operator std::string ( ) const;

	virtual object::ObjectBase * inc ( ) &&;
};

template < class FirstType, class SecondType >
ObjectsPair < FirstType, SecondType >::ObjectsPair(ext::pair<FirstType, SecondType> raw) : ext::pair<FirstType, SecondType>(std::move(raw)) {

}

template < class FirstType, class SecondType >
ObjectsPair < FirstType, SecondType >::ObjectsPair(FirstType firstObject, SecondType secondObject) : ext::pair<FirstType, SecondType>(std::move(firstObject), std::move(secondObject)) {

}

template < class FirstType, class SecondType >
ContainerBase* ObjectsPair < FirstType, SecondType >::clone ( ) const & {
	return new ObjectsPair(*this);
}

template < class FirstType, class SecondType >
ContainerBase* ObjectsPair < FirstType, SecondType >::clone() && {
	return new ObjectsPair(std::move(*this));
}

template < class FirstType, class SecondType >
int ObjectsPair < FirstType, SecondType >::compare(const ObjectsPair& other) const {
	static ext::compare<ext::pair<FirstType, SecondType>> comp;
	return comp ( static_cast < const ext::pair < FirstType, SecondType > & > ( * this ), static_cast < const ext::pair < FirstType, SecondType > & > ( other ) );
}

template < class FirstType, class SecondType >
void ObjectsPair < FirstType, SecondType >::operator>>(std::ostream& os) const {
	os << "(ObjectsPair " << static_cast < const ext::pair < FirstType, SecondType > & > ( * this ) << ")";
}

template < class FirstType, class SecondType >
ObjectsPair < FirstType, SecondType >::operator std::string() const {
	std::stringstream ss;
	ss << *this;
	return std::move(ss).str();
}

template < class FirstType, class SecondType >
object::ObjectBase* ObjectsPair < FirstType, SecondType >::inc() && {
	return new object::UniqueObject(object::Object(std::move(*this)), primitive::Integer(0));
}

} /* namespace container */

namespace core {

template < class FirstType, class SecondType >
struct normalize < container::ObjectsPair < FirstType, SecondType > > {
	static std::pair < object::Object, object::Object > raw2 ( std::pair < FirstType, SecondType > && source ) {
		return std::make_pair ( object::ObjectFactory::make ( std::move ( source.first ) ), object::ObjectFactory::make ( std::move ( source.second ) ) );
	}

	static ext::pair < object::Object, object::Object > raw ( ext::pair < FirstType, SecondType > && source ) {
		return ext::make_pair ( object::ObjectFactory::make ( std::move ( source.first ) ), object::ObjectFactory::make ( std::move ( source.second ) ) );
	}

	static container::ObjectsPair < > eval ( container::ObjectsPair < FirstType, SecondType > && value ) {
		return container::ObjectsPair < > ( raw ( std::move ( value ) ) );
	}
};

} /* namespace core */

#endif /* OBJECTS_PAIR_H_ */
