/*
 * Set.h
 *
 * Created on: Apr 1, 2013
 * Author: Jan Travnicek
 */

#ifndef OBJECTS_SET_H_
#define OBJECTS_SET_H_

#include <alib/set>
#include <alib/string>
#include <sstream>

#include "ContainerBase.h"

#include <object/UniqueObject.h>
#include <object/ObjectFactory.h>

#include <core/normalize.hpp>

namespace container {

/**
 * Basic container from which are derived all other containers.
 * Contains reason why the container occured.
 */
template < class ElementType = object::Object >
class ObjectsSet final : public ext::set < ElementType >, public ContainerBase {
public:
	explicit ObjectsSet ( ext::set < ElementType > );
	explicit ObjectsSet ( );

	virtual ContainerBase * clone ( ) const &;

	virtual ContainerBase * clone ( ) &&;

	virtual int compare ( const ObjectBase & other ) const {
		if ( ext::type_index ( typeid ( * this ) ) == ext::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return ext::type_index ( typeid ( * this ) ) - ext::type_index ( typeid ( other ) );
	}

	virtual int compare ( const ObjectsSet & other ) const;

	virtual void operator >>( std::ostream & os ) const;

	virtual explicit operator std::string ( ) const;

	virtual object::ObjectBase * inc ( ) &&;
};

template < class ElementType >
ObjectsSet < ElementType >::ObjectsSet(ext::set<ElementType> raw) : ext::set<ElementType>(std::move(raw)) {

}

template < class ElementType >
ObjectsSet < ElementType >::ObjectsSet() : ext::set<ElementType>() {

}

template < class ElementType >
ContainerBase* ObjectsSet < ElementType >::clone ( ) const & {
	return new ObjectsSet(*this);
}

template < class ElementType >
ContainerBase* ObjectsSet < ElementType >::clone() && {
	return new ObjectsSet(std::move(*this));
}

template < class ElementType >
int ObjectsSet < ElementType >::compare(const ObjectsSet& other) const {
	static ext::compare<ext::set<ElementType>> comp;
	return comp ( static_cast < const ext::set < ElementType > & > ( * this ), static_cast < const ext::set < ElementType > & > ( other ) );
}

template < class ElementType >
void ObjectsSet < ElementType >::operator>>(std::ostream& os) const {
	os << "(ObjectsSet " << static_cast < const ext::set < ElementType > & > ( * this ) << ")";
}

template < class ElementType >
ObjectsSet < ElementType >::operator std::string() const {
	std::stringstream ss;
	ss << *this;
	return std::move(ss).str();
}

template < class ElementType >
object::ObjectBase* ObjectsSet < ElementType >::inc() && {
	return new object::UniqueObject(object::Object(std::move(*this)), primitive::Integer(0));
}

} /* namespace container */

namespace core {

template < class ElementType >
struct normalize < container::ObjectsSet < ElementType > > {
	static ext::set < object::Object > raw ( ext::set < ElementType > && source ) {
		ext::set < object::Object > res;

		for ( ElementType && element : ext::make_moveable_set ( source ) )
			res.insert ( object::ObjectFactory::make ( std::move ( element ) ) );

		return res;
	}

	static container::ObjectsSet < > eval ( container::ObjectsSet < ElementType > && value ) {
		return container::ObjectsSet < > ( raw ( std::move ( value ) ) );
	}
};

} /* namespace core */

#endif /* OBJECTS_SET_H_ */
