/*
 * ObjectsTrie.h
 *
 * Created on: Apr 1, 2013
 * Author: Jan Travnicek
 */

#ifndef OBJECTS_TRIE_H_
#define OBJECTS_TRIE_H_

#include <alib/trie>
#include <alib/string>
#include <sstream>

#include "ContainerBase.h"

#include <object/UniqueObject.h>
#include <object/ObjectFactory.h>

#include <core/normalize.hpp>

namespace container {

/**
 * Basic container from which are derived all other containers.
 * Contains reason why the container occured.
 */
template < class KeyType = object::Object, class ValueType = object::Object >
class ObjectsTrie final : public ext::trie < KeyType, ValueType >, public ContainerBase {
public:
	explicit ObjectsTrie ( ext::trie < KeyType, ValueType > );

	virtual ContainerBase * clone ( ) const &;

	virtual ContainerBase * clone ( ) &&;

	virtual int compare ( const ObjectBase & other ) const {
		if ( ext::type_index ( typeid ( * this ) ) == ext::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return ext::type_index ( typeid ( * this ) ) - ext::type_index ( typeid ( other ) );
	}

	virtual int compare ( const ObjectsTrie & other ) const;

	virtual void operator >>( std::ostream & os ) const;

	virtual explicit operator std::string ( ) const;

	virtual object::ObjectBase * inc ( ) &&;
};

template < class KeyType, class ValueType >
ObjectsTrie < KeyType, ValueType >::ObjectsTrie ( ext::trie < KeyType, ValueType > raw ) : ext::trie < KeyType, ValueType > ( std::move ( raw ) ) {
}

template < class KeyType, class ValueType >
ContainerBase * ObjectsTrie < KeyType, ValueType >::clone ( ) const & {
	return new ObjectsTrie ( * this );
}

template < class KeyType, class ValueType >
ContainerBase * ObjectsTrie < KeyType, ValueType >::clone ( ) && {
	return new ObjectsTrie ( std::move ( * this ) );
}

template < class KeyType, class ValueType >
int ObjectsTrie < KeyType, ValueType >::compare ( const ObjectsTrie & other ) const {
	static ext::compare < ext::trie < KeyType, ValueType > > comp;

	return comp ( static_cast < const ext::trie < KeyType, ValueType > & > ( * this ), static_cast < const ext::trie < KeyType, ValueType > & > ( other ) );
}

template < class KeyType, class ValueType >
void ObjectsTrie < KeyType, ValueType >::operator >>( std::ostream & os ) const {
	os << "(ObjectsTrie " << static_cast < const ext::trie < KeyType, ValueType > & > ( * this ) << ")";
}

template < class KeyType, class ValueType >
ObjectsTrie < KeyType, ValueType >::operator std::string ( ) const {
	std::stringstream ss;
	ss << * this;
	return std::move ( ss ).str ( );
}

template < class KeyType, class ValueType >
object::ObjectBase* ObjectsTrie < KeyType, ValueType >::inc() && {
	return new object::UniqueObject(object::Object(std::move(*this)), primitive::Integer(0));
}

} /* namespace container */

namespace core {

template < class KeyType, class ValueType >
struct normalize < container::ObjectsTrie < KeyType, ValueType > > {
	static ext::trie < object::Object, object::Object > raw ( ext::trie < KeyType, ValueType > && source ) {
		ext::map < object::Object, ext::trie < object::Object, object::Object > > children;

		for ( std::pair < KeyType, ext::trie < KeyType, ValueType > > && child : ext::make_moveable_map ( source.getChildren ( ) ) ) {
			children.insert ( std::make_pair ( object::ObjectFactory::make ( std::move ( child.first ) ), raw ( std::move ( child.second ) ) ) );
		}

		return ext::trie < object::Object, object::Object > ( object::ObjectFactory::make ( std::move ( source.getData ( ) ) ), std::move ( children ) );
	}

	static container::ObjectsTrie < > eval ( container::ObjectsTrie < KeyType, ValueType > && value ) {
		return container::ObjectsTrie < > ( raw ( std::move ( value ) ) );
	}
};

} /* namespace core */

#endif /* OBJECTS_TRIE_H_ */
