/*
 * vector.h
 *
 * Created on: Apr 1, 2013
 * Author: Jan Travnicek
 */

#ifndef OBJECTS_VECTOR_H_
#define OBJECTS_VECTOR_H_

#include <alib/vector>
#include <alib/string>
#include <sstream>

#include "ContainerBase.h"

#include <object/UniqueObject.h>
#include <object/ObjectFactory.h>

#include <core/normalize.hpp>

namespace container {

/**
 * Basic container from which are derived all other containers.
 * Contains reason why the container occured.
 */
template < class ElementType = object::Object >
class ObjectsVector final : public ext::vector < ElementType >, public ContainerBase {
public:
	explicit ObjectsVector ( ext::vector < ElementType > );
	explicit ObjectsVector ( );

	virtual ContainerBase * clone ( ) const &;

	virtual ContainerBase * clone ( ) &&;

	virtual int compare ( const ObjectBase & other ) const {
		if ( ext::type_index ( typeid ( * this ) ) == ext::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return ext::type_index ( typeid ( * this ) ) - ext::type_index ( typeid ( other ) );
	}

	virtual int compare ( const ObjectsVector & other ) const;

	virtual void operator >>( std::ostream & os ) const;

	virtual explicit operator std::string ( ) const;

	virtual object::ObjectBase * inc ( ) &&;
};

template < class ElementType >
ObjectsVector < ElementType >::ObjectsVector ( ext::vector < ElementType > raw ) : ext::vector < ElementType > ( std::move ( raw ) ) {
}

template < class ElementType >
ObjectsVector < ElementType >::ObjectsVector ( ) : ext::vector < ElementType > ( ) {
}

template < class ElementType >
ContainerBase * ObjectsVector < ElementType >::clone ( ) const & {
	return new ObjectsVector ( * this );
}

template < class ElementType >
ContainerBase * ObjectsVector < ElementType >::clone ( ) && {
	return new ObjectsVector ( std::move ( * this ) );
}

template < class ElementType >
int ObjectsVector < ElementType >::compare ( const ObjectsVector & other ) const {
	static ext::compare < ext::vector < ElementType > > comp;

	return comp ( static_cast < const ext::vector < ElementType > & > ( * this ), static_cast < const ext::vector < ElementType > & > ( other ) );
}

template < class ElementType >
void ObjectsVector < ElementType >::operator >>( std::ostream & os ) const {
	os << "(ObjectsVector " << static_cast < const ext::vector < ElementType > & > ( * this ) << ")";
}

template < class ElementType >
ObjectsVector < ElementType >::operator std::string ( ) const {
	std::stringstream ss;
	ss << * this;
	return std::move ( ss ).str ( );
}

template < class ElementType >
object::ObjectBase* ObjectsVector < ElementType >::inc() && {
	return new object::UniqueObject(object::Object(std::move(*this)), primitive::Integer(0));
}

} /* namespace container */

namespace core {

template < class ElementType >
struct normalize < container::ObjectsVector < ElementType > > {
	static ext::vector < object::Object > raw ( ext::vector < ElementType > && source ) {
		ext::vector < object::Object > res;

		for ( ElementType & element : source )
			res.push_back ( object::ObjectFactory::make ( std::move ( element ) ) );

		return res;
	}

	static container::ObjectsVector < > eval ( container::ObjectsVector < ElementType > && value ) {
		return container::ObjectsVector < > ( raw ( std::move ( value ) ) );
	}
};

} /* namespace core */

#endif /* OBJECTS_VECTOR_H_ */
