/*
 * ObjectFactory.h
 *
 *  Created on: Mar 16, 2018
 *      Author: Jan Travnicek
 */

#ifndef _OBJECT_FACTORY_H__
#define _OBJECT_FACTORY_H__

#include "Object.h"
#include <alib/variant>
#include <object/AnyObject.h>

namespace object {

class ObjectFactory {
	template < class Variant, class Type, class ... Types >
	static Object processVariant ( Variant && data ) {
		if ( data.template is < Type > ( ) ) {
			return make ( std::move ( data.template get < Type > ( ) ) );
		} else {
			return processVariant < Variant, Types ... > ( std::move ( data ) );
		}
	}

	template < class Variant >
	static Object processVariant ( Variant && ) {
		throw std::logic_error ( "Variant state error" );
	}

public:
	static Object make ( bool boolean );
	static Object make ( char character );
	static Object make ( double number );
	static Object make ( int number );
	static Object make ( std::string string );
	static Object make ( const char * string );
	static Object make ( unsigned number );

	static Object make ( Object other ) {
		return other;
	}

	template < class Type, typename std::enable_if < ! std::is_convertible < Type *, object::ObjectBase * >::value, Type >::type * = nullptr >
	static Object make ( Type && data ) {
		return Object ( object::AnyObject < Type > ( std::move ( data ) ) );
	}

	static Object make ( object::ObjectBase && data ) {
		return Object ( std::move ( data ) );
	}

	template < class ... Types >
	static Object make ( ext::variant < Types ... > && data ) {
		return processVariant < ext::variant < Types ... >, Types ... > ( std::move ( data ) );
	}
};

} /* namespace object */

#endif /* _OBJECT_FACTORY_H__ */
