/*
 * LabeledSymbol.h
 *
 * This file is part of Algorithms library toolkit.
 * Copyright (C) 2017 Jan Travnicek (jan.travnicek@fit.cvut.cz)

 * Algorithms library toolkit is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * Algorithms library toolkit is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with Algorithms library toolkit.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Created on: Mar 26, 2013
 *      Author: Martin Zak
 */

#ifndef LABELED_SYMBOL_H_
#define LABELED_SYMBOL_H_

#include <label/Label.h>
#include "SymbolBase.h"

namespace alphabet {

/**
 * \brief
 * \depricated
 * Represents labeled symbol used as a wrapper of label.
 */
class LabeledSymbol : public SymbolBase {
	/**
	 * \brief
	 * The label of the symbol.
	 */
	label::Label m_label;

public:
	/**
	 * \brief
	 * Creates a new instance of the symbol with integer label.
	 */
	explicit LabeledSymbol ( int number );

	/**
	 * \brief
	 * Creates a new instance of the symbol with character label.
	 */
	explicit LabeledSymbol ( char character );

	/**
	 * \brief
	 * Creates a new instance of the symbol with string label.
	 */
	explicit LabeledSymbol ( std::string label );

	/**
	 * \brief
	 * Creates a new instance of the symbol with preexistent label.
	 */
	explicit LabeledSymbol ( label::Label label );

	/**
	 * @copydoc alphabet::SymbolBase::clone ( ) const &
	 */
	virtual SymbolBase * clone ( ) const &;

	/**
	 * @copydoc alphabet::SymbolBase::clone ( ) &&
	 */
	virtual SymbolBase * clone ( ) &&;

	/**
	 * Getter of the symbol's label.
	 *
	 * \return the label of the symbol
	 */
	const label::Label & getLabel ( ) const &;

	/**
	 * Getter of the symbol's label.
	 *
	 * \return the label of the symbol
	 */
	label::Label && getLabel ( ) &&;

	/**
	 * @copydoc base::CommonBase < ObjectBase >::compare ( const ObjectBase & ) const
	 */
	virtual int compare ( const ObjectBase & other ) const {
		if ( ext::type_index ( typeid ( * this ) ) == ext::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return ext::type_index ( typeid ( * this ) ) - ext::type_index ( typeid ( other ) );
	}

	/**
	 * The actual compare method
	 *
	 * \param other the other instance
	 *
	 * \returns the actual relation between two by type same symbols
	 */
	virtual int compare ( const LabeledSymbol & other ) const;

	/**
	 * @copydoc base::CommonBase < ObjectBase >::operator >> ( std::ostream & ) const
	 */
	virtual void operator >>( std::ostream & out ) const;

	/**
	 * @copydoc base::CommonBase < ObjectBase >::operator std::string ( ) const
	 */
	virtual explicit operator std::string ( ) const;

	/**
	 * @copydoc object::ObjectBase::inc ( ) &&
	 */
	virtual SymbolBase * inc ( ) &&;
};

} /* namespace alphabet */

#endif /* LABELED_SYMBOL_H_ */
