/*
 * FailStateLabel.h
 *
 * This file is part of Algorithms library toolkit.
 * Copyright (C) 2017 Jan Travnicek (jan.travnicek@fit.cvut.cz)

 * Algorithms library toolkit is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * Algorithms library toolkit is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with Algorithms library toolkit.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#ifndef FAIL_STATE_LABEL_H_
#define FAIL_STATE_LABEL_H_

#include "LabelBase.h"
#include "Label.h"
#include <object/Object.h>

namespace label {

/**
 * \brief
 * Represents label of the fail state of an automaton.
 */
class FailStateLabel final : public LabelBase {
public:
	/**
	 * \brief
	 * Creates a new instance of the label.
	 */
	explicit FailStateLabel ( );

	/**
	 * @copydoc label::LabelBase::clone ( ) const &
	 */
	virtual LabelBase * clone ( ) const &;

	/**
	 * @copydoc label::LabelBase::clone ( ) &&
	 */
	virtual LabelBase * clone ( ) &&;

	/**
	 * @copydoc base::CommonBase < ObjectBase >::compare ( const ObjectBase & ) const
	 */
	virtual int compare ( const ObjectBase & other ) const {
		if ( ext::type_index ( typeid ( * this ) ) == ext::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return ext::type_index ( typeid ( * this ) ) - ext::type_index ( typeid ( other ) );
	}

	/**
	 * The actual compare method
	 *
	 * \param other the other instance
	 *
	 * \returns the actual relation between two by type same labels
	 */
	virtual int compare ( const FailStateLabel & other ) const;

	/**
	 * @copydoc base::CommonBase < ObjectBase >::operator >> ( std::ostream & ) const
	 */
	virtual void operator >>( std::ostream & ) const;

	/**
	 * @copydoc base::CommonBase < ObjectBase >::operator std::string ( ) const
	 */
	virtual explicit operator std::string ( ) const;

	/**
	 * \brief Factory for the label construction of the label based on given type.
	 */
	template < typename Base >
	static inline typename std::enable_if <   std::is_integral < Base >::value, Base >::type instance ( );

	/**
	 * \brief Factory for the label construction of the label based on given type.
	 */
	template < typename Base >
	static inline typename std::enable_if < ! std::is_integral < Base >::value, Base >::type instance ( );

	/**
	 * @copydoc object::ObjectBase::inc ( ) &&
	 */
	virtual LabelBase * inc ( ) &&;
};

template < typename Base >
inline typename std::enable_if < std::is_integral < Base >::value, Base >::type FailStateLabel::instance ( ) {
	return -1;
}

template < >
inline std::string FailStateLabel::instance < std::string > ( ) {
	return std::string ( "fail" );
}

template < >
inline FailStateLabel FailStateLabel::instance < FailStateLabel > ( ) {
	return FailStateLabel ( );
}

template < >
inline Label FailStateLabel::instance < Label > ( ) {
	return Label ( FailStateLabel ( ) );
}

template < >
inline object::Object FailStateLabel::instance < object::Object > ( ) {
	return object::Object ( FailStateLabel ( ) );
}

template < typename Base >
inline typename std::enable_if < ! std::is_integral < Base >::value, Base >::type FailStateLabel::instance ( ) {
	return { };
}

} /* namespace label */

#endif /* FAIL_STATE_LABEL_H_ */
