/*
 * PrimitiveLabel.cpp
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travicek
 */

#include "PrimitiveLabel.h"
#include "UniqueLabel.h"

#include <registration/ValuePrinterRegistration.hpp>

namespace label {

PrimitiveLabel::PrimitiveLabel(primitive::Primitive primitive) : m_primitive(std::move(primitive)) {

}

LabelBase* PrimitiveLabel::clone ( ) const & {
	return new PrimitiveLabel(*this);
}

LabelBase* PrimitiveLabel::clone() && {
	return new PrimitiveLabel(std::move(*this));
}

const primitive::Primitive& PrimitiveLabel::getData() const {
	return m_primitive;
}

primitive::Primitive& PrimitiveLabel::getData() {
	return m_primitive;
}

int PrimitiveLabel::compare(const PrimitiveLabel& other) const {
	return m_primitive.getData().compare(other.m_primitive.getData());
}

void PrimitiveLabel::operator>>(std::ostream& out) const {
	out << "(PrimitiveLabel " << m_primitive << ")";
}

PrimitiveLabel::operator std::string() const {
	return (std::string) m_primitive;
}

LabelBase* PrimitiveLabel::inc() && {
	m_primitive.inc();
	return NULL;
}

} /* namespace label */

namespace {

static auto valuePrinter = registration::ValuePrinterRegister < label::PrimitiveLabel > ( );

} /* namespace */
