/*
 * PrimitiveLabel.h
 *
 * This file is part of Algorithms library toolkit.
 * Copyright (C) 2017 Jan Travnicek (jan.travnicek@fit.cvut.cz)

 * Algorithms library toolkit is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * Algorithms library toolkit is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with Algorithms library toolkit.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#ifndef PRIMITIVE_LABEL_H_
#define PRIMITIVE_LABEL_H_

#include <primitive/Primitive.h>
#include "LabelBase.h"

namespace label {

/**
 * \brief
 * \depricated
 * Represents a label serving as wrapper of a primitive datatype (int, unsigned, char, etc.)
 */
class PrimitiveLabel final : public LabelBase {
	/**
	 * \brief
	 * The primitive value of the label.
	 */
	primitive::Primitive m_primitive;

public:
	/**
	 * \brief
	 * Creates a new instance of the label with preexistent primitive.
	 */
	explicit PrimitiveLabel ( primitive::Primitive primitive );

	/**
	 * @copydoc alphabet::SymbolBase::clone ( ) const &
	 */
	virtual LabelBase * clone ( ) const &;

	/**
	 * @copydoc alphabet::SymbolBase::clone ( ) &&
	 */
	virtual LabelBase * clone ( ) &&;

	/**
	 * Getter of the label's primitive value.
	 *
	 * \return the primitive value of the label
	 */
	const primitive::Primitive & getData ( ) const;

	/**
	 * Getter of the label's primitive value.
	 *
	 * \return the primitive value of the label
	 */
	primitive::Primitive & getData ( );

	/**
	 * @copydoc base::CommonBase < ObjectBase >::compare ( const ObjectBase & ) const
	 */
	virtual int compare ( const ObjectBase & other ) const {
		if ( ext::type_index ( typeid ( * this ) ) == ext::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return ext::type_index ( typeid ( * this ) ) - ext::type_index ( typeid ( other ) );
	}

	/**
	 * The actual compare method
	 *
	 * \param other the other instance
	 *
	 * \returns the actual relation between two by type same labels
	 */
	virtual int compare ( const PrimitiveLabel & other ) const;

	/**
	 * @copydoc base::CommonBase < ObjectBase >::operator >> ( std::ostream & ) const
	 */
	virtual void operator >>( std::ostream & ) const;

	/**
	 * @copydoc base::CommonBase < ObjectBase >::operator std::string ( ) const
	 */
	virtual explicit operator std::string ( ) const;

	/**
	 * @copydoc object::ObjectBase::inc ( ) &&
	 */
	virtual LabelBase * inc ( ) &&;
};

} /* namespace label */

#endif /* PRIMITIVE_LABEL_H_ */
