/*
 * RegExp.h
 *
 *  Created on: Apr 10, 2013
 *      Author: Martin Zak
 */

#ifndef REG_EXP_H_
#define REG_EXP_H_

#include <base/WrapperBase.hpp>
#include "RegExpBase.h"

#include <common/DefaultSymbolType.h>
#include <alib/string>
#include <alib/vector>
#include <object/ObjectFactory.h>

#include "unbounded/UnboundedRegExpSymbol.h"
#include "unbounded/UnboundedRegExpEmpty.h"
#include "unbounded/UnboundedRegExpConcatenation.h"

#include "unbounded/UnboundedRegExp.h"
#include "unbounded/UnboundedRegExpStructure.h"

#include <string/LinearString.h>

namespace regexp {

/**
 * Wrapper around automata.
 */
class RegExp : public base::WrapperBase < RegExpBase > {
	using base::WrapperBase < RegExpBase >::WrapperBase;

};

template < class T >
using SymbolTypeOfRegExp = typename std::decay < decltype (std::declval<T>().getAlphabet()) >::type::value_type;

regexp::RegExp regexpFrom ( const std::string & string );

regexp::RegExp regexpFrom ( const char * string );

template < class SymbolType >
regexp::RegExp regexpFrom ( ext::vector < SymbolType > string ) {
	regexp::UnboundedRegExpConcatenation < DefaultSymbolType > con;

	for ( auto & symbol : string )
		con.appendElement ( regexp::UnboundedRegExpSymbol < DefaultSymbolType > ( object::ObjectFactory::make ( symbol ) ) );

	return regexp::RegExp { regexp::UnboundedRegExp < > ( regexp::UnboundedRegExpStructure < DefaultSymbolType > ( std::move ( con ) ) ) };
}

template < class SymbolType >
regexp::RegExp regexpFrom ( string::LinearString < SymbolType > string ) {
	return regexpFrom ( string.getContent ( ) );
}

template < class SymbolType >
regexp::RegExp regexpFrom ( const SymbolType & symbol ) {
	return regexp::RegExp { regexp::UnboundedRegExp < > ( regexp::UnboundedRegExpStructure < DefaultSymbolType > ( regexp::UnboundedRegExpSymbol < DefaultSymbolType > ( object::ObjectFactory::make ( symbol ) ) ) ) };
}


regexp::RegExp regexpFrom ( );

} /* namespace regexp */

#endif /* REG_EXP_H_ */
