/*
 * array.hpp
 *
 * Created on: Apr 1, 2013
 * Author: Jan Travnicek
 */

#ifndef __ARRAY_HPP_
#define __ARRAY_HPP_

#include <array>
#include <experimental/array>

#include <ostream>
#include <sstream>
#include <string>

#include "compare.hpp"

namespace ext {

template < class T, std::size_t N >
class array : public std::array < T, N > {
public:
#ifdef __clang__
	using std::array < T, N >::array;
	using std::array < T, N >::operator =;
#else
	array ( ) noexcept : std::array < T, N > ( ) {
	}

	array ( const array & other ) = default;

	array ( array && other ) = default;

	using std::array < T, N >::array;

	array & operator = ( array && other ) = default;

	array & operator = ( const array & other ) = default;

	using std::array < T, N >::operator =;
#endif

	template < class ... Types >
	array ( Types && ... args ) : std::array < T, N > ( std::experimental::make_array ( std::forward < Types > ( args ) ... ) ) {
	}
};

template< class T, std::size_t N >
std::ostream& operator<<(std::ostream& out, const ext::array < T, N > & array) {
	out << "[";

	bool first = true;
	for(const T& item : array) {
		if(!first) out << ", ";
		first = false;
		out << item;
	}

	out << "]";
	return out;
}

template< class T, std::size_t N >
struct compare < ext::array < T, N > > {
	int operator ( ) ( const ext::array < T, N > & first, const ext::array < T, N > & second) const {
		static compare<typename std::decay < T >::type > comp;
		for(auto iterF = first.begin(), iterS = second.begin(); iterF != first.end(); ++iterF, ++iterS) {
			int res = comp(*iterF, *iterS);
			if(res != 0) return res;
		}
		return 0;
	}
};

template< class T, std::size_t N >
std::string to_string ( const ext::array < T, N > & value ) {
	std::stringstream ss;
	ss << value;
	return ss.str();
}

template < typename Base, typename ... Types >
constexpr array < typename std::remove_reference < Base >::type, sizeof ... ( Types ) + 1 > make_array ( Base && first, Types && ... other ) {
	return array < typename std::remove_reference < Base >::type, sizeof ... ( Types ) + 1 > ( std::forward < Base > ( first ), std::forward < Types > ( other ) ... );
}

template < typename Base >
constexpr array < typename std::remove_reference < Base >::type, 0 > make_array ( ) {
	return array < typename std::remove_reference < Base >::type, 0 > ( );
}

} /* namespace ext */

#endif /* __ARRAY_HPP_ */
