/*
 * compare.hpp
 *
 * Created on: Apr 1, 2013
 * Author: Jan Travnicek
 */

#ifndef __COMPARE_HPP_
#define __COMPARE_HPP_

#include <utility>

namespace ext {

template < typename T, typename Enable = void >
struct compare;

template<>
struct compare<bool> {
	int operator()(bool first, bool second) const {
		return first - second;
	}
};

template<>
struct compare<unsigned char> {
	int operator()(unsigned char first, unsigned char second) const {
		return first - second;
	}
};

template<>
struct compare<signed char> {
	int operator()(signed char first, signed char second) const {
		return first - second;
	}
};

template<>
struct compare<char> {
	int operator()(char first, char second) const {
		return first - second;
	}
};

template<>
struct compare<unsigned short> {
	int operator()(unsigned short first, unsigned short second) const {
		return first - second;
	}
};

template<>
struct compare<signed short> {
	int operator()(signed short first, signed short second) const {
		return first - second;
	}
};

template<>
struct compare<unsigned int> {
	int operator()(unsigned int first, unsigned int second) const {
		return first - second;
	}
};

template<>
struct compare<signed int> {
	int operator()(signed int first, signed int second) const {
		return first - second;
	}
};

template<>
struct compare<signed long> {
	int operator()(signed long first, signed long second) const {
		if ( first < second )
			return -1;
		else if ( first > second )
			return 1;
		else
			return 0;
	}
};

template<>
struct compare<unsigned long> {
	int operator()(unsigned long first, unsigned long second) const {
		if ( first < second )
			return -1;
		else if ( first > second )
			return 1;
		else
			return 0;
	}
};

template<>
struct compare<signed long long> {
	int operator()(signed long long first, signed long long second) const {
		if ( first < second )
			return -1;
		else if ( first > second )
			return 1;
		else
			return 0;
	}
};

template<>
struct compare<unsigned long long> {
	int operator()(unsigned long long first, unsigned long long second) const {
		if ( first < second )
			return -1;
		else if ( first > second )
			return 1;
		else
			return 0;
	}
};

template<>
struct compare<float> {
	int operator()(float first, float second) const {
		if ( first < second )
			return -1;
		else if ( first > second )
			return 1;
		else
			return 0;
	}
};

template<>
struct compare<double> {
	int operator()(double first, double second) const {
		if ( first < second )
			return -1;
		else if ( first > second )
			return 1;
		else
			return 0;
	}
};

template<>
struct compare<long double> {
	int operator()(long double first, long double second) const {
		if ( first < second )
			return -1;
		else if ( first > second )
			return 1;
		else
			return 0;
	}
};

template<class T>
struct compare<T*> {
	int operator()( T * const & first, T * const & second) const {
		static compare<typename std::decay < T >::type > comp;
		return comp(*first, *second);
	}
};

template<class T>
struct compare<T&> {
	int operator()( const T & first, const T & second) const {
		static compare<typename std::decay < T >::type > comp;
		return comp(first, second);
	}
};

} /* namespace ext */

#endif /* __COMPARE_HPP_ */
