/*
 * deque.hpp
 *
 * Created on: Apr 1, 2013
 * Author: Jan Travnicek
 */

#ifndef __DEQUE_HPP_
#define __DEQUE_HPP_

#include <deque>
#include <ostream>
#include <sstream>
#include <string>

#include "compare.hpp"
#include "allocFix.hpp"

namespace ext {

template < class T, class Alloc = std::allocator < T > >
class deque : public std::deque < T, Alloc >, AllocFix < Alloc > {
public:
#ifdef __clang__
	using std::deque < T, Alloc >::deque;
	using std::deque < T, Alloc >::operator =;
#else
	deque ( ) noexcept : std::deque < T, Alloc > ( ) {
	}

	deque ( const deque & other ) = default;

	deque ( deque && other ) = default;

	using std::deque < T, Alloc >::deque;

	deque & operator = ( deque && other ) = default;

	deque & operator = ( const deque & other ) = default;

	using std::deque < T, Alloc >::operator =;
#endif
};

template < class T, class ... Ts >
std::ostream & operator <<( std::ostream & out, const ext::deque < T, Ts ... > & list ) {
	out << "[";

	bool first = true;

	for ( const T & item : list ) {
		if ( !first ) out << ", ";

		first = false;
		out << item;
	}

	out << "]";
	return out;
}

template < class T, class ... Ts >
struct compare < ext::deque < T, Ts ... > > {
	int operator ()( const ext::deque < T, Ts ... > & first, const ext::deque < T, Ts ... > & second ) const {
		if ( first.size ( ) < second.size ( ) ) return -1;

		if ( first.size ( ) > second.size ( ) ) return 1;

		static compare < typename std::decay < T >::type > comp;

		for ( auto iterF = first.begin ( ), iterS = second.begin ( ); iterF != first.end ( ); ++iterF, ++iterS ) {
			int res = comp ( * iterF, * iterS );

			if ( res != 0 ) return res;
		}

		return 0;
	}

};

template < class T, class ... Ts >
std::string to_string ( const ext::deque < T, Ts ... > & value ) {
	std::stringstream ss;
	ss << value;
	return ss.str();
}

} /* namespace ext */

#endif /* __DEQUE_HPP_ */
