/*
 * forward_list.hpp
 *
 * Created on: Apr 1, 2013
 * Author: Jan Travnicek
 */

#ifndef __FORWARD_LIST_HPP_
#define __FORWARD_LIST_HPP_

#include <forward_list>
#include <ostream>
#include <sstream>
#include <string>

#include "compare.hpp"
#include "allocFix.hpp"

namespace ext {

template < class T, class Alloc = std::allocator < T > >
class forward_list : public std::forward_list < T, Alloc >, AllocFix < Alloc > {
public:
#ifdef __clang__
	using std::forward_list < T, Alloc >::forward_list;
	using std::forward_list < T, Alloc >::operator =;
#else
	forward_list ( ) noexcept : std::forward_list < T, Alloc > ( ) {
	}

	forward_list ( const forward_list & other ) noexcept : std::forward_list < T, Alloc > ( other ) {
	}

	forward_list ( forward_list && other ) noexcept : std::forward_list < T, Alloc > ( std::move ( other ) ) {
	}

	using std::forward_list < T, Alloc >::forward_list;

	forward_list & operator = ( forward_list && other ) noexcept {
		static_cast < std::forward_list < T, Alloc > & > ( * this ) = std::move ( other );
		return * this;
	}

	forward_list & operator = ( const forward_list & other ) noexcept {
		static_cast < std::forward_list < T, Alloc > & > ( * this ) = other;
		return * this;
	}

	using std::forward_list < T, Alloc >::operator =;
#endif
};

template< class T, class ... Ts >
std::ostream& operator<<(std::ostream& out, const ext::forward_list<T, Ts ... >& forward_list) {
	out << "[";

	bool first = true;
	for(const T& item : forward_list) {
		if(!first) out << ", ";
		first = false;
		out << item;
	}

	out << "]";
	return out;
}

template<class T, class ... Ts >
struct compare<ext::forward_list<T, Ts ... >> {
	int operator()(const ext::forward_list<T, Ts ... >& first, const ext::forward_list<T, Ts ... >& second) const {
		if(first.size() < second.size()) return -1;
		if(first.size() > second.size()) return 1;

		static compare<typename std::decay < T >::type > comp;
		for(auto iterF = first.begin(), iterS = second.begin(); iterF != first.end(); ++iterF, ++iterS) {
			int res = comp(*iterF, *iterS);
			if(res != 0) return res;
		}
		return 0;
	}
};

template < class T, class ... Ts >
std::string to_string ( const ext::forward_list < T, Ts ... > & value ) {
	std::stringstream ss;
	ss << value;
	return ss.str();
}

} /* namespace ext */

#endif /* __FORWARD_LIST_HPP_ */

